@@ -88,7 +88,7 @@
  * Basic Server model:
  *  1. call sasl_server_init() at startup to load plug-ins
  *  2. On connection, call sasl_server_new()
- * [3. call sasl_listmech() and send list to client]
+ *  3. call sasl_listmech() and send list to client]
  *  4. after client AUTH command, call sasl_server_start(), goto 5a
  *  5. call sasl_server_step()
  *  5a. If SASL_CONTINUE, output to client, wait response, repeat 5
@@ -123,7 +123,7 @@
 
 #define SASL_VERSION_MAJOR 2
 #define SASL_VERSION_MINOR 0
-#define SASL_VERSION_STEP 3
+#define SASL_VERSION_STEP 4
 
 #include "prop.h"
 #define LIBSASL_API
@@ -258,7 +258,6 @@ typedef unsigned sasl_ssf_t;
 
 /* usage flags provided to sasl_server_new and sasl_client_new:
  */
-#define SASL_CLIENT_FALLBACK 0x0002 /* allow client to fallback */
 #define SASL_SUCCESS_DATA    0x0004 /* server supports data on success */
 
 /***************************
@@ -318,11 +317,19 @@ typedef struct sasl_security_properties
  * Callback types *
  ******************/
 
-/* Extensible type for a client/server callbacks
+/*
+ * Extensible type for a client/server callbacks
  *  id      -- identifies callback type
  *  proc    -- procedure call arguments vary based on id
  *  context -- context passed to procedure
  */
+/* Note that any memory that is allocated by the callback needs to be
+ * freed by the application, be it via function call or interaction.
+ *
+ * It may be freed after sasl_*_step returns SASL_OK.  if the mechanism
+ * requires this information to persist (for a security layer, for example)
+ * it must maintain a private copy.
+ */
 typedef struct sasl_callback {
     /* Identifies the type of the callback function.
      * Mechanisms must ignore callbacks with id's they don't recognize.
@@ -565,6 +572,12 @@ typedef int sasl_server_userdb_setpass_t(sasl_conn_t *conn,
 #define SASL_CB_SERVER_USERDB_SETPASS (0x8006)
 
 /* callback for a server-supplied user canonicalization function.
+ *
+ * This function is called directly after the mechanism has the
+ * authentication and authorization IDs.  It is called before any
+ * User Canonicalization plugin is called.  It has the responsibility
+ * of copying its output into the provided output buffers.
+ * 
  *  user, ulen    -- user name (authorization name), may not be NUL terminated
  *                   may be same as out_user
  *  authid, alen  -- authentication name, may not be NUL terminated
