@@ -1,7 +1,7 @@
 /* common.c - Functions that are common to server and clinet
  * Rob Siemborski
  * Tim Martin
- * $Id: common.c,v 1.64.2.48 2001/07/30 17:23:30 rjs3 Exp $
+ * $Id: common.c,v 1.64.2.57 2001/08/24 23:24:36 rbraun Exp $
  */
 /* 
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
@@ -70,8 +70,6 @@
 #include <unistd.h>
 #endif
 
-#include <sys/uio.h> /* for struct iovec */
-
 static const char build_ident[] = "$Build: libsasl " PACKAGE "-" VERSION " $";
 
 void *free_mutex = NULL;
@@ -137,7 +135,7 @@ int _sasl_strdup(const char *in, char **out, int *outlen)
 }
 
 /* adds a string to the buffer; reallocing if need be */
-static int add_string(char **out, int *alloclen, int *outlen, char *add)
+int _sasl_add_string(char **out, int *alloclen, int *outlen, const char *add)
 {
   int addlen;
 
@@ -193,6 +191,12 @@ int sasl_encodev(sasl_conn_t *conn,
     if (! invec || ! output || ! outputlen || numiov < 1)
 	PARAMERROR(conn);
 
+    if(!conn->props.maxbufsize) {
+	sasl_seterror(conn, 0,
+		      "called sasl_encode[v] with application that does not support security layers");
+	return SASL_TOOWEAK;
+    }
+
     /* This might be better to check on a per-plugin basis, but I think
      * it's cleaner and more effective here.  It also encourages plugins
      * to be honest about what they accept */
@@ -229,6 +233,12 @@ int sasl_decode(sasl_conn_t *conn,
     if(!input || !output || !outputlen)
 	PARAMERROR(conn);
 
+    if(!conn->props.maxbufsize) {
+	sasl_seterror(conn, 0,
+		      "called sasl_decode with application that does not support security layers");
+	return SASL_TOOWEAK;
+    }
+
     if(inputlen > conn->oparams.maxoutbuf)
 	PARAMERROR(conn);
     
@@ -515,12 +525,24 @@ int sasl_getprop(sasl_conn_t *conn, int propnum, const void **pvalue)
   case SASL_SERVICE:
       *((const char **)pvalue) = conn->service;
       break;
-  case SASL_AUTHSOURCE: /* FIXME: I doubt these are identical */
-  case SASL_MECHNAME:
+  case SASL_AUTHSOURCE: /* name of plugin (not name of mech) */
       if(conn->type == SASL_CONN_CLIENT) {
-	  *((const char **)pvalue) = ((sasl_client_conn_t *)conn)->mech->plug->mech_name;
+	  *((const char **)pvalue) =
+	      ((sasl_client_conn_t *)conn)->mech->plugname;
       } else if (conn->type == SASL_CONN_SERVER) {
-	  *((const char **)pvalue) = ((sasl_server_conn_t *)conn)->mech->plug->mech_name;
+	  *((const char **)pvalue) =
+	      ((sasl_server_conn_t *)conn)->mech->plugname;
+      } else {
+	  result = SASL_BADPARAM;
+      }
+      break;
+  case SASL_MECHNAME: /* name of mech */
+      if(conn->type == SASL_CONN_CLIENT) {
+	  *((const char **)pvalue) =
+	      ((sasl_client_conn_t *)conn)->mech->plug->mech_name;
+      } else if (conn->type == SASL_CONN_SERVER) {
+	  *((const char **)pvalue) =
+	      ((sasl_server_conn_t *)conn)->mech->plug->mech_name;
       } else {
 	  result = SASL_BADPARAM;
       }
@@ -604,6 +626,25 @@ int sasl_setprop(sasl_conn_t *conn, int propnum, const void *value)
 	  strcpy(conn->ipremoteport, ipremoteport);
 	  conn->got_ip_remote = 1;
       }
+      
+      if(conn->got_ip_remote) {
+	  if(conn->type == SASL_CONN_CLIENT) {
+	      ((sasl_client_conn_t *)conn)->cparams->ipremoteport
+		  = conn->ipremoteport;
+	  } else if (conn->type == SASL_CONN_SERVER) {
+	      ((sasl_server_conn_t *)conn)->sparams->ipremoteport
+		  = conn->ipremoteport;
+	  }
+      } else {
+	  if(conn->type == SASL_CONN_CLIENT) {
+	      ((sasl_client_conn_t *)conn)->cparams->ipremoteport
+		  = NULL;
+	  } else if (conn->type == SASL_CONN_SERVER) {
+	      ((sasl_server_conn_t *)conn)->sparams->ipremoteport
+		  = NULL;
+	  }
+      }
+
       break;
   }
 
@@ -620,6 +661,24 @@ int sasl_setprop(sasl_conn_t *conn, int propnum, const void *value)
 	  strcpy(conn->iplocalport, iplocalport);
 	  conn->got_ip_local = 1;
       }
+
+      if(conn->got_ip_local) {
+	  if(conn->type == SASL_CONN_CLIENT) {
+	      ((sasl_client_conn_t *)conn)->cparams->iplocalport
+		  = conn->iplocalport;
+	  } else if (conn->type == SASL_CONN_SERVER) {
+	      ((sasl_server_conn_t *)conn)->sparams->iplocalport
+		  = conn->iplocalport;
+	  }
+      } else {
+	  if(conn->type == SASL_CONN_CLIENT) {
+	      ((sasl_client_conn_t *)conn)->cparams->iplocalport
+		  = NULL;
+	  } else if (conn->type == SASL_CONN_SERVER) {
+	      ((sasl_server_conn_t *)conn)->sparams->iplocalport
+		  = NULL;
+	  }
+      }
       break;
   }
 
@@ -709,176 +768,6 @@ const char *sasl_errdetail(sasl_conn_t *conn)
 }
 
 
-/* set the error string which will be returned by sasl_errdetail() using  
- *  syslog()-style formatting (e.g. printf-style with %m as most recent
- *  errno error)
- * 
- *  primarily for use by server callbacks such as the sasl_authorize_t
- *  callback and internally to plug-ins
- *
- * This will also trigger a call to the SASL logging callback (if any)
- * with a level of SASL_LOG_FAIL unless the SASL_NOLOG flag is set.
- *
- * Messages should be sensitive to the current language setting.  If there
- * is no SASL_CB_LANGUAGE callback messages MUST be US-ASCII otherwise UTF-8
- * is used and use of RFC 2482 for mixed-language text is encouraged.
- * 
- * if conn is NULL, function does nothing
- */
-void sasl_seterror(sasl_conn_t *conn,
-		   unsigned flags,
-		   const char *fmt, ...) 
-{
-  int outlen=0; /* current length of output buffer */
-  int pos=0; /* current position in format string */
-  int formatlen;
-  int result;
-  sasl_log_t *log_cb;
-  void *log_ctx;
-  int ival;
-  char *cval;
-  va_list ap; /* varargs thing */
-
-  if(!conn) {
-      if(!(flags & SASL_NOLOG)) {
-	  /* See if we have a logging callback... */
-	  result = _sasl_getcallback(NULL, SASL_CB_LOG, &log_cb, &log_ctx);
-	  if (result == SASL_OK && ! log_cb)
-	      result = SASL_FAIL;
-	  if (result != SASL_OK)
-	      return;
-	  
-	  log_cb(log_ctx, SASL_LOG_FAIL,
-		 "No sasl_conn_t passed to sas_seterror");
-      }  
-      return;
-  } else if(!fmt) return;    
-
-  formatlen = strlen(fmt);
-
-  va_start(ap, fmt); /* start varargs */
-
-  while(pos<formatlen)
-  {
-    if (fmt[pos]!='%') /* regular character */
-    {
-      result = _buf_alloc(&conn->error_buf, &conn->error_buf_len, outlen+1);
-      if (result != SASL_OK)
-	return;
-      conn->error_buf[outlen]=fmt[pos];
-      outlen++;
-      pos++;
-    } else { /* formating thing */
-      int done=0;
-      char frmt[10];
-      int frmtpos=1;
-      char tempbuf[21];
-      frmt[0]='%';
-      pos++;
-
-      while (done==0)
-      {
-	switch(fmt[pos])
-	  {
-	  case 's': /* need to handle this */
-	    cval = va_arg(ap, char *); /* get the next arg */
-	    result = add_string(&conn->error_buf, &conn->error_buf_len,
-				&outlen, cval);
-	      
-	    if (result != SASL_OK) /* add the string */
-	      return;
-
-	    done=1;
-	    break;
-
-	  case '%': /* double % output the '%' character */
-	    result = _buf_alloc(&conn->error_buf,&conn->error_buf_len,
-				outlen+1);
-	    if (result != SASL_OK)
-	      return;
-	    conn->error_buf[outlen]='%';
-	    outlen++;
-	    done=1;
-	    break;
-
-	  case 'm': /* insert the errno string */
-	    result = add_string(&conn->error_buf, &conn->error_buf_len,
-				&outlen, strerror(va_arg(ap, int)));
-	    if (result != SASL_OK)
-	      return;
-	    done=1;
-	    break;
-
-	  case 'z': /* insert the sasl error string */
-	    result = add_string(&conn->error_buf, &conn->error_buf_len,
-				&outlen,
-				(char *)sasl_errstring(sasl_usererr(
-				    va_arg(ap, int)),NULL,NULL));
-	    if (result != SASL_OK)
-	      return;
-	    done=1;
-	    break;
-
-	  case 'c':
-	    frmt[frmtpos++]=fmt[pos];
-	    frmt[frmtpos]=0;
-	    tempbuf[0] = (char) va_arg(ap, int); /* get the next arg */
-	    tempbuf[1]='\0';
-	    
-	    /* now add the character */
-	    result = add_string(&conn->error_buf, &conn->error_buf_len,
-				&outlen, tempbuf);
-	    if (result != SASL_OK)
-	      return;
-	    done=1;
-	    break;
-
-	  case 'd':
-	  case 'i':
-	    frmt[frmtpos++]=fmt[pos];
-	    frmt[frmtpos]=0;
-	    ival = va_arg(ap, int); /* get the next arg */
-
-	    snprintf(tempbuf,20,frmt,ival); /* have snprintf do the work */
-	    /* now add the string */
-	    result = add_string(&conn->error_buf, &conn->error_buf_len,
-				&outlen, tempbuf);
-	    if (result != SASL_OK)
-	      return;
-	    done=1;
-
-	    break;
-	  default: 
-	    frmt[frmtpos++]=fmt[pos]; /* add to the formating */
-	    frmt[frmtpos]=0;	    
-	    if (frmtpos>9) 
-	      done=1;
-	  }
-	pos++;
-	if (pos>formatlen)
-	  done=1;
-      }
-
-    }
-  }
-
-  conn->error_buf[outlen]='\0'; /* put 0 at end */
-
-  va_end(ap);  
-  
-  if(!(flags & SASL_NOLOG)) {
-      /* See if we have a logging callback... */
-      result = _sasl_getcallback(conn, SASL_CB_LOG, &log_cb, &log_ctx);
-      if (result == SASL_OK && ! log_cb)
-	  result = SASL_FAIL;
-      if (result != SASL_OK)
-	  return;
-      
-      result = log_cb(log_ctx, SASL_LOG_FAIL, conn->error_buf);
-  }
-}
-
-
 static int
 _sasl_global_getopt(void *context,
 		    const char *plugin_name,
@@ -1251,7 +1140,7 @@ _sasl_log (sasl_conn_t *conn,
 	  {
 	  case 's': /* need to handle this */
 	    cval = va_arg(ap, char *); /* get the next arg */
-	    result = add_string(&out, &alloclen,
+	    result = _sasl_add_string(&out, &alloclen,
 				&outlen, cval);
 	      
 	    if (result != SASL_OK) /* add the string */
@@ -1271,7 +1160,7 @@ _sasl_log (sasl_conn_t *conn,
 	    break;
 
 	  case 'm': /* insert the errno string */
-	    result = add_string(&out, &alloclen, &outlen,
+	    result = _sasl_add_string(&out, &alloclen, &outlen,
 				strerror(va_arg(ap, int)));
 	    if (result != SASL_OK)
 		goto done;
@@ -1280,7 +1169,7 @@ _sasl_log (sasl_conn_t *conn,
 	    break;
 
 	  case 'z': /* insert the sasl error string */
-	    result = add_string(&out, &alloclen, &outlen,
+	    result = _sasl_add_string(&out, &alloclen, &outlen,
 				(char *) sasl_errstring(va_arg(ap, int),NULL,NULL));
 	    if (result != SASL_OK)
 		goto done;
@@ -1295,7 +1184,7 @@ _sasl_log (sasl_conn_t *conn,
 	    tempbuf[1]='\0';
 	    
 	    /* now add the character */
-	    result = add_string(&out, &alloclen, &outlen, tempbuf);
+	    result = _sasl_add_string(&out, &alloclen, &outlen, tempbuf);
 	    if (result != SASL_OK)
 		goto done;
 		
@@ -1310,7 +1199,7 @@ _sasl_log (sasl_conn_t *conn,
 
 	    snprintf(tempbuf,20,frmt,ival); /* have snprintf do the work */
 	    /* now add the string */
-	    result = add_string(&out, &alloclen, &outlen, tempbuf);
+	    result = _sasl_add_string(&out, &alloclen, &outlen, tempbuf);
 	    if (result != SASL_OK)
 		goto done;
 
@@ -1405,6 +1294,7 @@ _sasl_alloc_utils(sasl_conn_t *conn,
 
   utils->seterror=&sasl_seterror;
 
+#ifndef macintosh
   /* Aux Property Utilities */
   utils->prop_new=&prop_new;
   utils->prop_dup=&prop_dup;
@@ -1417,6 +1307,7 @@ _sasl_alloc_utils(sasl_conn_t *conn,
   utils->prop_set=&prop_set;
   utils->prop_setvals=&prop_setvals;
   utils->prop_erase=&prop_erase;
+#endif
 
   /* Spares */
   utils->spare_fptr = NULL;
@@ -1537,6 +1428,14 @@ int _buf_alloc(char **rwbuf, unsigned *curlen, unsigned newlen)
     return SASL_OK;
 }
 
+/* for the mac os x cfm glue: this lets the calling function
+   get pointers to the error buffer without having to touch the sasl_conn_t struct */
+void _sasl_get_errorbuf(sasl_conn_t *conn, char ***bufhdl, unsigned **lenhdl)
+{
+	*bufhdl = &conn->error_buf;
+	*lenhdl = &conn->error_buf_len;
+}
+
 /* convert an iovec to a single buffer */
 int _iovec_to_buf(const struct iovec *vec,
 		  unsigned numiov, buffer_info_t **output) 
@@ -1575,6 +1474,8 @@ int _iovec_to_buf(const struct iovec *vec,
     return SASL_OK;
 }
 
+/* This code might be useful in the future, but it isn't now, so.... */
+#if 0
 int _sasl_iptostring(const struct sockaddr *addr, socklen_t addrlen,
 		     char *out, unsigned outlen) {
     char hbuf[NI_MAXHOST], pbuf[NI_MAXSERV];
@@ -1591,12 +1492,13 @@ int _sasl_iptostring(const struct sockaddr *addr, socklen_t addrlen,
 
     return SASL_OK;
 }
+#endif
 
 int _sasl_ipfromstring(const char *addr,
 		       struct sockaddr *out, socklen_t outlen) 
 {
     int i, j;
-    struct addrinfo hints, *ai;
+    struct addrinfo hints, *ai = NULL;
     char hbuf[NI_MAXHOST];
     
     /* A NULL out pointer just implies we don't do a copy, just verify it */
@@ -1626,8 +1528,10 @@ int _sasl_ipfromstring(const char *addr,
 	return SASL_BADPARAM;
 
     if (out) {
-	if (outlen < (unsigned)ai->ai_addrlen)
+	if (outlen < (unsigned)ai->ai_addrlen) {
+	    freeaddrinfo(ai);
 	    return SASL_BUFOVER;
+	}
 	memcpy(out, ai->ai_addr, ai->ai_addrlen);
     }
 
