@@ -1,7 +1,7 @@
 /* SASL server API implementation
  * Rob Siemborski
  * Tim Martin
- * $Id: server.c,v 1.84.2.52 2001/07/25 15:32:06 rjs3 Exp $
+ * $Id: server.c,v 1.84.2.57 2001/08/17 16:49:17 rjs3 Exp $
  */
 /* 
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
@@ -275,7 +275,7 @@ static int init_mechlist(void)
  * parameters:
  *  p - entry point
  */
-int sasl_server_add_plugin(const char *plugname __attribute__((unused)),
+int sasl_server_add_plugin(const char *plugname,
 			   sasl_server_plug_init_t *p)
 {
     int plugcount;
@@ -286,7 +286,7 @@ int sasl_server_add_plugin(const char *plugname __attribute__((unused)),
     int version;
     int lupe;
 
-    if(!p) return SASL_BADPARAM;
+    if(!plugname || !p) return SASL_BADPARAM;
 
     entry_point = (sasl_server_plug_init_t *)p;
 
@@ -315,6 +315,10 @@ int sasl_server_add_plugin(const char *plugname __attribute__((unused)),
 	if (! mech) return SASL_NOMEM;
 
 	mech->plug=pluglist++;
+	if(_sasl_strdup(plugname, &mech->plugname, NULL) != SASL_OK) {
+	    sasl_FREE(mech);
+	    return SASL_NOMEM;
+	}
 	mech->version = version;
 
 	/* wheather this mech actually has any users in it's db */
@@ -349,7 +353,8 @@ static void server_done(void) {
 	  if (prevm->plug->glob_context!=NULL) {
 	      sasl_FREE(prevm->plug->glob_context);
 	  }
-	  	  
+
+	  sasl_FREE(prevm->plugname);	  	  
 	  sasl_FREE(prevm);    
       }
       _sasl_free_utils(&mechlist->utils);
@@ -627,7 +632,7 @@ int sasl_server_init(const sasl_callback_t *callbacks,
     }
 
     /* load internal plugins */
-    sasl_server_add_plugin(NULL, &external_server_init);
+    sasl_server_add_plugin("EXTERNAL", &external_server_init);
 
     /* delayed loading of plugins? (DSO only, as it doesn't
      * make much [any] sense to delay in the static library case) */
@@ -756,14 +761,6 @@ int sasl_server_new(const char *service,
 
   memset(*pconn, 0, sizeof(sasl_server_conn_t));
 
-  (*pconn)->destroy_conn = &server_dispose;
-  result = _sasl_conn_init(*pconn, service, flags, SASL_CONN_SERVER,
-			   &server_idle, serverFQDN,
-			   iplocalport, ipremoteport,
-			   callbacks, &global_callbacks);
-  if (result != SASL_OK)
-      goto done_error;
-
   serverconn = (sasl_server_conn_t *)*pconn;
 
   /* make sparams */
@@ -773,6 +770,15 @@ int sasl_server_new(const char *service,
 
   memset(serverconn->sparams, 0, sizeof(sasl_server_params_t));
 
+  (*pconn)->destroy_conn = &server_dispose;
+  result = _sasl_conn_init(*pconn, service, flags, SASL_CONN_SERVER,
+			   &server_idle, serverFQDN,
+			   iplocalport, ipremoteport,
+			   callbacks, &global_callbacks);
+  if (result != SASL_OK)
+      goto done_error;
+
+
   /* set util functions - need to do rest */
   utils=_sasl_alloc_utils(*pconn, &global_callbacks);
   if (!utils) {
@@ -796,19 +802,6 @@ int sasl_server_new(const char *service,
   serverconn->sparams->props = serverconn->base.props;
 
   /* set some variables */
-
-  if((*pconn)->got_ip_local) {      
-     serverconn->sparams->iplocalport = (*pconn)->iplocalport;
-  } else {
-     serverconn->sparams->iplocalport = NULL;
-  }
-
-  if((*pconn)->got_ip_remote) {
-      serverconn->sparams->ipremoteport = (*pconn)->ipremoteport;
-  } else {
-      serverconn->sparams->ipremoteport = NULL;
-  }
-
   if (user_realm) {
     result = _sasl_strdup(user_realm, &serverconn->user_realm, NULL);
   } else {
@@ -946,7 +939,7 @@ static int mech_permitted(sasl_conn_t *conn,
 
     /* do we want to special case SASL_SEC_PASS_CREDENTIALS? nah.. */
     if (((myflags ^ plug->security_flags) & myflags) != 0) {
-	sasl_seterror(conn, 0,
+	sasl_seterror(conn, SASL_NOLOG,
 		      "security flags do not match required");
 	return 0;
     }
@@ -1236,7 +1229,7 @@ int sasl_server_step(sasl_conn_t *conn,
 	    ret = SASL_CONTINUE;
 	}
 	if(!conn->oparams.maxoutbuf) {
-	    conn->oparams.maxoutbuf = DEFAULT_MAXOUTBUF;
+	    conn->oparams.maxoutbuf = conn->props.maxbufsize;
 	}
     }
     
@@ -1491,20 +1484,34 @@ int sasl_user_exists(sasl_conn_t *conn,
 		     const char *user) 
 {
     int result=SASL_NOMECH;
+    const char *mech;
+    void *context;
+    sasl_getopt_t *getopt;
     struct sasl_verify_password_s *v;
-
+    
     /* check params */
     if (_sasl_server_active==0) return SASL_NOTINIT;
     if (!conn) return SASL_BADPARAM;
-    
-    if(!user) 
+    if (!user || conn->type != SASL_CONN_SERVER) 
 	PARAMERROR(conn);
 
     if(!service) service = conn->service;
+    
+    /* figure out how to check (i.e. auxprop or saslauthd or pwcheck) */
+    if (_sasl_getcallback(conn, SASL_CB_GETOPT, &getopt, &context)
+            == SASL_OK) {
+        getopt(context, NULL, "pwcheck_method", &mech, NULL);
+    }
+
+    if(!mech) mech = DEFAULT_CHECKPASS_MECH;
+
+    result = SASL_NOMECH;
 
     for (v = _sasl_verify_password; v->name; v++) {
-	result = v->verify(conn, user, NULL, service, user_realm);
-	if(result == SASL_BADPARAM || result == SASL_OK) break;
+	if(is_mech(mech, v->name)) {
+	    result = v->verify(conn, user, NULL, service, user_realm);
+	    break;
+	}
     }
 
     /* Screen out the SASL_BADPARAM response
