@@ -16,6 +16,10 @@
 
 #include "plugin_common.h"
 
+#ifdef macintosh
+#include <sasl_crammd5_plugin_decl.h>
+#endif
+
 SASL_CLIENT_PLUG_INIT( crammd5 )
 SASL_SERVER_PLUG_INIT( crammd5 )
 
