@@ -1,7 +1,7 @@
 /* Kerberos4 SASL plugin
  * Rob Siemborski
  * Tim Martin 
- * $Id: kerberos4.c,v 1.65.2.36 2001/07/27 20:48:01 rjs3 Exp $
+ * $Id: kerberos4.c,v 1.65.2.42 2001/08/24 23:26:49 rbraun Exp $
  */
 /* 
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
@@ -69,7 +69,6 @@
 
 #include <errno.h>
 #include <ctype.h>
-#include <sys/uio.h>
 
 #include "plugin_common.h"
 
@@ -113,6 +112,10 @@ static const char rcsid[] = "$Implementation: Carnegie Mellon SASL " VERSION " $
 extern int gethostname(char *, int);
 #endif
 
+#ifndef KEYFILE
+#define KEYFILE "/etc/srvtab";
+#endif
+
 #define KRB_SECFLAG_NONE (1)
 #define KRB_SECFLAG_INTEGRITY (2)
 #define KRB_SECFLAG_ENCRYPTION (4)
@@ -342,7 +345,7 @@ static int kerberosv4_decode_once(void *context,
 
     /* see if the krb library gave us a failure */
     if (result != 0) {
-	text->utils->seterror(text->utils->conn, 0, krb_err_txt[result]);
+	text->utils->seterror(text->utils->conn, 0, get_krb_err_txt(result));
 	return SASL_FAIL;
     }
 
@@ -623,7 +626,7 @@ static int kerberosv4_server_mech_step (void *conn_context,
     if (result) { /* if fails mechanism fails */
 	text->utils->seterror(text->utils->conn,0,
 	"krb_rd_req failed service=%s instance=%s error code=%s (%i)",
-	    sparams->service, text->instance,krb_err_txt[result],result);
+	    sparams->service, text->instance,get_krb_err_txt(result),result);
 	return SASL_BADAUTH;
     }
 
@@ -842,24 +845,21 @@ static int kerberosv4_server_mech_step (void *conn_context,
   return SASL_FAIL; /* should never get here */
 }
 
-int mech_avail(void *glob_context __attribute__((unused)),
-#ifndef KRB4_IGNORE_IP_ADDRESS
-	       sasl_server_params_t *sparams,
-#else
-	       sasl_server_params_t *sparams __attribute__((unused)),
-#endif
-	       void **conn_context __attribute__((unused))) 
+static int mech_avail(void *glob_context __attribute__((unused)),
+		      sasl_server_params_t *sparams,
+		      void **conn_context __attribute__((unused))) 
 {
-#ifndef KRB4_IGNORE_IP_ADDRESS
-    /* FIXME: this should also ensure that it is an IPv4 address! */
-    if(!sparams->iplocalport || !sparams->ipremoteport
-	|| strchr(sparams->iplocalport, ':')
-	|| strchr(sparams->ipremoteport, ':')) {
+    struct sockaddr_in addr;
+
+    if (!sparams->iplocalport || !sparams->ipremoteport
+       || ipv4_ipfromstring(sparams->utils,
+                            sparams->iplocalport, &addr) != SASL_OK
+       || ipv4_ipfromstring(sparams->utils,
+                            sparams->ipremoteport, &addr) != SASL_OK) {
 	SETERROR(sparams->utils,
 		 "KERBEROS_V4 unavailable due to lack of IPv4 information");
 	return SASL_NOMECH;
     }
-#endif
 
     return SASL_OK;
 }
@@ -926,7 +926,10 @@ int kerberos4_server_plug_init(const sasl_utils_t *utils,
     if (access(srvtab, R_OK) != 0) {
 	utils->log(NULL, SASL_LOG_ERR,
 		   "can't access srvtab file %s: %m", srvtab, errno);
-	utils->free(srvtab);
+	if(!(--refcount)) {
+	    utils->free(srvtab);
+	    srvtab=NULL;
+	}
 	return SASL_FAIL;
     }
 
@@ -1012,7 +1015,7 @@ static int kerberosv4_client_mech_step(void *conn_context,
 	memset(&text->credentials,0,sizeof(text->credentials));
 	if(kcglue_krb_mk_req(ticket.dat,
 			     &ticket.length,
-			     params->service,
+			     cparams->service,
 			     text->instance,
 			     text->realm,
 			     text->challenge,
@@ -1028,7 +1031,7 @@ static int kerberosv4_client_mech_step(void *conn_context,
 	    
 	    cparams->utils->log(NULL, SASL_LOG_ERR, 
 			       "krb_mk_req() failed: %s (%d)",
-			       krb_err_txt[result], result);
+			       get_krb_err_txt(result), result);
 	    return SASL_FAIL;
 	}
 
@@ -1179,7 +1182,7 @@ static int kerberosv4_client_mech_step(void *conn_context,
 	if(result != 0) {
 	    cparams->utils->log(NULL, SASL_LOG_ERR,
 				"krb_get_cred() failed: %s (%d)",
-				krb_err_txt[result], result);
+				get_krb_err_txt(result), result);
 	    SETERROR(cparams->utils, "krb_get_cred() failed");
 	    return SASL_BADAUTH;
 	}
